// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "assert/assert.hpp"
#include "Camera.hpp"
#include "game/Game.hpp"

using namespace Dawn;

void Camera::onInit() {
  this->onResizeListener = this->getRenderTarget()->onResize.listen([&](
    float_t width, float_t height
  ) {
    this->onResize.emit(this->getRenderTarget(), width, height);
  });
}

void Camera::onDispose() {
  renderTarget = nullptr;
}

std::shared_ptr<RenderTarget> Camera::getRenderTarget() {
  if(this->renderTarget) return this->renderTarget;
  return getGame()->renderHost.getBackBufferRenderTarget();
}

glm::mat4 Camera::getProjection() {
  switch(this->type) {
    case CameraType::ORTHOGONAL:
      return glm::ortho(
        (float_t)this->orthoLeft,
        (float_t)this->orthoRight,
        (float_t)this->orthoBottom,
        (float_t)this->orthoTop,
        (float_t)this->clipNear,
        (float_t)this->clipFar
      );

    case CameraType::PERSPECTIVE:
      return glm::perspective(
        (float_t)this->fov,
        this->getAspect(),
        (float_t)this->clipNear,
        (float_t)this->clipFar
      );
  }

  assertUnreachable("Invalid Camera Type!");
  return glm::mat4(1.0f);
}

float_t Camera::getAspect() {
  auto rt = this->getRenderTarget();
  return rt->getWidth() / rt->getHeight();
}

void Camera::setRenderTarget(std::shared_ptr<RenderTarget> renderTarget) {
  onResizeListener();
  this->renderTarget = renderTarget;
  this->onResizeListener = this->getRenderTarget()->onResize.listen([&](
    float_t width, float_t height
  ) {
    this->onResize.emit(this->getRenderTarget(), width, height);
  });
}