// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PokerTurn.hpp"
#include "PokerPlayer.hpp"
#include "PokerGame.hpp"

using namespace Dawn;

struct PokerTurn PokerTurn::bet(
  std::shared_ptr<PokerPlayer> player,
  const int32_t chips
) {
  struct PokerTurn turn;
  int32_t i;

  assertNotNull(player, "Player cannot be null.");
  assertTrue(chips >= 0, "Chips must be greater than or equal to 0.");

  turn.player = player;
  turn.confidence = 1;

  if(chips == 0) {
    turn.type = PokerTurnType::Check;
    turn.chips = 0;
  } else if(player->chips <= chips) {
    turn.chips = player->chips;
    turn.type = PokerTurnType::AllIn;
  } else {
    turn.chips = chips;
    turn.type = PokerTurnType::Bet;

    auto pg = player->pokerGame.lock();
    assertNotNull(pg, "Player must be in a game.");

    i = pg->getCurrentCallValue();

    if(chips == (i - player->currentBet)) turn.type = PokerTurnType::Call;
  }

  return turn;
}

struct PokerTurn PokerTurn::fold(std::shared_ptr<PokerPlayer> player) {
  struct PokerTurn turn;
  turn.player = player;
  turn.chips = 0;
  turn.confidence = 1;
  turn.type = PokerTurnType::Fold;
  return turn;
}

void PokerTurn::action() {
  assertNotNull(this->player, "Player cannot be null.");

  switch(this->type) {
    case PokerTurnType::Bet:
    case PokerTurnType::Call:
    case PokerTurnType::AllIn:
      this->player->bet(this->chips);
      break;

    case PokerTurnType::Check:
      player->bet(0);
      break;

    case PokerTurnType::Fold:
      player->fold();
      break;
    
    default:
      assertUnreachable("Unknown turn type.");
      break;
  }
}