// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  class PokerPlayer;

  enum class PokerTurnType : uint8_t {
    Out = 0,
    Fold = 1,
    Bet = 2,
    Call = 3,
    Check = 4,
    AllIn = 5
  };

  struct PokerTurn {
    public:
      /** What type of action the turn is */
      enum PokerTurnType type;
      /** How many chips they did in their turn (if applicable) */
      int32_t chips;
      /** How confident the AI is about their turn. 0 = none, 1 = full */
      float_t confidence;
      /** Player that this action belongs to */
      std::shared_ptr<PokerPlayer> player;

      /**
       * Generate a turn action for betting as a player.
       * 
       * @param player Player index who is betting.
       * @param chips Chips to raise by.
       * @return A turn for a bet action.
       */
      static struct PokerTurn bet(
        std::shared_ptr<PokerPlayer> player, const int32_t chips
      );

      /**
       * Return a turn action for the given player to fold.
       * 
       * @return A turn for a fold action.
       */
      static struct PokerTurn fold(std::shared_ptr<PokerPlayer> player);

      /**
       * Actions / Performs this turn against the defined player.
       */
      void action();
  };
}