// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "IntervalEvent.hpp"

using namespace Dawn;

IntervalEvent::IntervalEvent() : CustomEvent() {
  internalData = 0.0f;
}

enum CustomEventResult IntervalEvent::shouldEmit(
  const struct IntervalEventData &listener
) {
  if(listener.nextInvoke > internalData) return CustomEventResult::NOTHING;
  return CustomEventResult::INVOKE;
}

struct IntervalEventData IntervalEvent::transformData(const float_t &interval) {
  
  return {
    .interval = interval,
    .nextInvoke = internalData + interval
  };
}

struct IntervalEventData IntervalEvent::transformDataAfterEmit(
  const struct IntervalEventData &listenerData
) {
  return {
    .interval = listenerData.interval,
    .nextInvoke = listenerData.nextInvoke + listenerData.interval
  };
}

void IntervalEvent::tick(const float_t delta) {
  internalData += delta;
  emit();
}