
// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "scene/SceneList.hpp"
#include "component/display/Camera.hpp"
#include "prefab/SimpleSpinningCube.hpp"
#include "display/font/TrueTypeTexture.hpp"
#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "display/mesh/QuadMesh.hpp"

#include "component/ui/UICanvas.hpp"
#include "ui/elements/UIRectangle.hpp"
#include "ui/elements/UILabel.hpp"
#include "ui/UIMenu.hpp"
#include "ui/container/UIRowContainer.hpp"
#include "ui/container/UIPaddingContainer.hpp"

#include <ft2build.h>
#include FT_FREETYPE_H

using namespace Dawn;

std::shared_ptr<TrueTypeTexture> texture;

void Dawn::helloWorldScene(Scene &s) {
  std::string font = "ysabeau_medium";
  texture = s.getGame()->assetManager.get<TrueTypeTexture>(font, 24);

  while(!s.getGame()->assetManager.isLoaded(font)) {
    s.getGame()->assetManager.update();
  }

  auto cameraItem = s.createSceneItem();
  auto camera = cameraItem->addComponent<Camera>();
  cameraItem->lookAt({ 120, 0, 300 }, { 120, 0, 0 }, { 0, 1, 0 });
  camera->clipFar = 99999.99f;

  // auto quad = s.createSceneItem();
  // auto quadMesh = std::make_shared<Mesh>();

  // glm::vec2 position = { 0, 0 };
  // glm::vec2 size = texture->bufferStringToMesh(
  //   quadMesh,
  //   L"Hello World!",
  //   position,
  //   true  
  // );

  // auto quadRenderer = quad->addComponent<MeshRenderer>();
  // quadRenderer->mesh = quadMesh;

  // auto quadMaterial = quad->addComponent<SimpleTexturedMaterial>();
  // quadMaterial->setTexture(texture->texture);

  auto uiCanvasItem = s.createSceneItem();
  auto uiCanvas = uiCanvasItem->addComponent<UICanvas>();

  auto container = std::make_shared<UIContainer>();
  container->align = { 32, 32, UI_ALIGN_SIZE_AUTO, UI_ALIGN_SIZE_AUTO };
  container->alignX = UIAlignmentType::START;
  container->alignY = UIAlignmentType::START;
  uiCanvas->addElement(container);

  auto rect = std::make_shared<UIRectangle>();
  rect->color = COLOR_MAGENTA;
  container->appendChild(rect);

  auto menu = std::make_shared<UIMenu>();
  menu->setSize(1, 4);
  container->appendChild(menu);

  auto rowContainer = std::make_shared<UIRowContainer>();
  container->appendChild(rowContainer);

  std::vector<std::wstring> labels = {
    L"New Game",
    L"Load Game",
    L"Options",
    L"Exit"
  };

  for(auto &label : labels) {
    auto padding = std::make_shared<UIPaddingContainer>();
    padding->align = { 0, 0, UI_ALIGN_SIZE_AUTO, UI_ALIGN_SIZE_AUTO };
    padding->alignX = UIAlignmentType::START;
    padding->alignY = UIAlignmentType::START;
    padding->padding = { 8, 8, 8, 8 };

    auto labelElement = std::make_shared<UILabel>();
    labelElement->wordWrap = false;
    labelElement->align = { 0, 0, UI_ALIGN_SIZE_AUTO, UI_ALIGN_SIZE_AUTO };
    labelElement->setText(label);
    labelElement->setFont(texture);
    
    padding->appendChild(labelElement);
    rowContainer->appendChild(padding);
  }
}