// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "main.hpp"
#include "assert/assert.hpp"
#include "display/RenderHost.hpp"
#include "game/Game.hpp"
#include "asset/AssetDataLoader.hpp"
#include "environment/Environment.hpp"
#include <filesystem>

using namespace Dawn;
namespace fs = std::filesystem;

FILE * AssetDataLoader::openAssetArchiveFile() {
  auto path = environment.getVariable("assetsPath");
  auto file = fopen(path.c_str(), "rb");
  if(!file) {
    assertUnreachable("Failed to open assets file.\n%s!", strerror(errno));
    return nullptr;
  }
  return file;
}

int32_t main(int32_t argc, const char **argv) {
  //Set the path
  assertTrue(argc > 0, "No executable path provided.");
  environment.setVariable("executablePath", argv[0]);

  auto execPath = fs::path(environment.getVariable("executablePath"));
  auto execDir = execPath;
  while(fs::is_directory(execDir)) execDir = execDir.parent_path();
  environment.setVariable("executableDir", execDir.string());

  // Find the assets
  std::vector<fs::path> pathsToTryAndFindAssets = {
    execDir / "assets.tar",
    execDir / "assets.zip",
    execDir / "data" / "assets.tar",
    execDir / "data" / "assets.zip",
    execDir.parent_path() / "assets.tar",
    execDir.parent_path() / "assets.zip",
    execDir.parent_path() / "data" / "assets.tar",
    execDir.parent_path() / "data" / "assets.zip",
    execDir.parent_path().parent_path() / "assets.tar",
    execDir.parent_path().parent_path() / "assets.zip",
    execDir.parent_path().parent_path() / "data" / "assets.tar",
    execDir.parent_path().parent_path() / "data" / "assets.zip"
  };
  auto matchingPathIfAny = std::find_if(pathsToTryAndFindAssets.begin(), pathsToTryAndFindAssets.end(), [](const fs::path &p) {
    return fs::exists(p);
  });

  if(matchingPathIfAny == pathsToTryAndFindAssets.end()) {
    std::cout << "Could not find game assets" << std::endl;
    return 1;
  }
  environment.setVariable("assetsPath", matchingPathIfAny->string());

  //Create the game
  auto game = std::make_shared<Game>();
  game->init();

  while(!game->isCloseRequested()) {
    game->update();
  }

  return 0;
}